@@ -18,11 +18,12 @@ while( <CURL> )
 }
 close CURL;
 
+$curl_protocols =~ s/\r//;
 $curl_protocols =~ /\w+: (.*)$/;
 @curl = split / /,$1;
 
 # These features are not supported by curl-config
-@curl = grep(!/^(Debug|TrackMemory|Largefile|CharConv|GSS-Negotiate|SPNEGO)$/i, @curl);
+@curl = grep(!/^(Debug|TrackMemory|Metalink|Largefile|CharConv|GSS-Negotiate|SPNEGO)$/i, @curl);
 @curl = sort @curl;
 
 # Read the output of curl-config
@@ -31,7 +32,10 @@ open(CURLCONFIG, "sh $ARGV[0] --$what|") || die "Can't get curl-config $what lis
 while( <CURLCONFIG> )
 {
     chomp;
-    push @curl_config, lc($_);
+    # ignore curl-config --features not in curl's feature list
+    if(!/^(GSS-API)$/) {
+        push @curl_config, lc($_);
+    }
 }
 close CURLCONFIG;
 
